/* db_vmfcnwin.c - intrinsic functions for windows
 *
 * Copyright (c) 2009 by David Michael Betz.  All rights reserved.
 *
 */

#include <stdlib.h>
#include "../runtime/db_vm.h"

void VM_fcninit(void)
{
}

void VM_fcnstop(void)
{
}

void fcn_waitforvsync(Interpreter *i)
{
	printf("waitforvsync\n");
	Drop(i, 1);
}

void fcn_gamepad(Interpreter *i)
{
	int which = i->sp[0];
	printf("gamepad: %d\n", which);
	Drop(i, 1);
}

void fcn_sound(Interpreter *i)
{
	int freq = i->sp[0];
	printf("sound: %d\n", freq);
	Drop(i, 2);
}

void fcn_plot(Interpreter *i)
{
	int x = i->sp[-2];
	int y = i->sp[-1];
	int color = i->sp[0];
    printf("plot: %d, %d, %d\n",x, y, color);
	Drop(i, 4);
}

void fcn_line(Interpreter *i)
{
	int x1 = i->sp[-4];
	int y1 = i->sp[-3];
	int x2 = i->sp[-2];
	int y2 = i->sp[-1];
	int color = i->sp[0];
    printf("line: %d, %d, %d, %d, %d\n", x1, y1, x2, y2, color);
	Drop(i, 6);
}

void fcn_hline(Interpreter *i)
{
	int x1 = i->sp[-3];
	int x2 = i->sp[-2];
	int y = i->sp[-1];
	int color = i->sp[0];
    printf("hline: %d, %d, %d, %d\n", x1, x2, y, color);
	Drop(i, 5);
}

void fcn_vline(Interpreter *i)
{
	int x = i->sp[-3];
	int y1 = i->sp[-2];
	int y2 = i->sp[-1];
	int color = i->sp[0];
    printf("vline: %d, %d, %d, %d\n", x, y1, y2, color);
	Drop(i, 5);
}

void fcn_clear(Interpreter *i)
{
	int color = i->sp[0];
    printf("clear: %d\n", color);
	Drop(i, 2);
}

void fcn_fillrect(Interpreter *i)
{
	int x = i->sp[-4];
	int y = i->sp[-3];
	int w = i->sp[-2];
	int h = i->sp[-1];
	int color = i->sp[0];
	printf("fillrect: %d, %d, %d, %d, %d\n", x, y, w, h, color);
	Drop(i, 6);
}

void fcn_circle(Interpreter *i)
{
	int x = i->sp[-3];
	int y = i->sp[-2];
	int r = i->sp[-1];
	int color = i->sp[0];
	printf("circle: %d, %d, %d, %d\n", x, y, r, color);
	Drop(i, 5);
}

void fcn_position(Interpreter *i)
{
    int x = i->sp[-1];
    int y = i->sp[0];
	printf("position (%d, %d)\n", x, y);
	Drop(i, 3);
}

void fcn_text(Interpreter *i)
{
    int x = i->sp[-2];
    int y = i->sp[-1];
	int inHeap;
    char *str = GetStringPtr(i, (uint16_t)i->sp[0], &inHeap);
    int len = GetStringLen(i, i->sp[0]);
	printf("text (%d, %d): ", x, y);
    while (--len >= 0)
        putchar(VMGETBYTE(str++, inHeap));
    putchar('\n');
    StringRelease(i, i->sp[0]);
	Drop(i, 4);
}

void fcn_settilemap(Interpreter *i)
{
	int16_t *array = &i->variables[i->sp[0]];
    int16_t dim1 = array[-2];
    int16_t dim2 = array[-1] / dim1;
    VM_printf("settilemap (%d,%d)\n", dim1, dim2);
	Drop(i, 2);
}

void fcn_settiles(Interpreter *i)
{
	int16_t *array = &i->variables[i->sp[0]];
    int16_t dim1 = array[-2];
    int16_t dim2 = array[-1] / dim1;
    VM_printf("settiles (%d,%d)\n", dim1, dim2);
	Drop(i, 2);
}

void fcn_setcolortable(Interpreter *i)
{
	int16_t *array = &i->variables[i->sp[0]];
    int16_t dim1 = array[-2];
    int16_t dim2 = array[-1] / dim1;
    VM_printf("setcolortable (%d,%d)\n", dim1, dim2);
	Drop(i, 2);
}

void fcn_setsprite(Interpreter *i)
{
    int16_t n = i->sp[-4];
    int16_t state = i->sp[-3];
    int16_t x = i->sp[-2];
    int16_t y = i->sp[-1];
    int16_t tile = i->sp[0];
    VM_printf("setsprite: %d %d %d %d %d\n", n, state, x, y, tile);
    Drop(i, 6);
}
