#include <stdio.h>
#include "../runtime/db_vm.h"
#include "../common/db_packet.h"

Interpreter interp;

int main(int argc, char *argv[])
{
    Interpreter *i = &interp;

    VM_InitImage();
    InitInterpreter(i, TARGET_TILE);

    if (LoadImageFile(i, argv[1])) {
        if (i->state ==  STATE_STARTING) {
            while (Execute(i, 100) == STATE_RUNNING)
                ;
            VM_printf("\nBefore compact:\n");
            DumpHeap(i);
            CompactHeap(i);
            VM_printf("\nAfter compact:\n");
            DumpHeap(i);
        }
    }

    return 0;
}

/* VM_LoadMemoryImageHeader - load the header of an image already in memory */
int VM_LoadMemoryImageHeader(Interpreter *i)
{
    return VMFALSE;
}

void VM_printf(char *fmt, ...)
{
    va_list ap;
    va_start(ap, fmt);
    vprintf(fmt, ap);
    va_end(ap);
}

void VM_vprintf(char *fmt, va_list ap)
{
	vprintf(fmt, ap);
}

void VM_putchar(int ch)
{
	putchar(ch);
}

void VM_flush(void)
{
	fflush(stdout);
}

void VM_getline(void)
{
    char buf[PKTMAXLEN];
    if (fgets(buf, sizeof(buf), stdin))
        SetInputLine(buf, strlen(buf));
}

void VM_displaychar(int ch)
{
    putchar(ch);
}

void VM_displaystr(char *str)
{
    fputs(str, stdout);
}

void VM_displaytab(void)
{
}

VMFILE *VM_fopen(const char *name, const char *mode)
{
    return (VMFILE *)fopen(name, mode);
}

int VM_fclose(VMFILE *file)
{
    return fclose((FILE *)file);
}

size_t VM_fread(void *buf, size_t count, VMFILE *file)
{
    return fread(buf, 1, count, (FILE *)file);
}

size_t VM_fwrite(void *buf, size_t count, VMFILE *file)
{
    return fwrite(buf, 1, count, (FILE *)file);
}

int VM_fseek(VMFILE *file, long offset, int whence)
{
    return fseek((FILE *)file, offset, whence);
}

long VM_ftell(VMFILE *file)
{
    return ftell((FILE *)file);
}
