/* db_mdio.c - serial i/o routines
 *
 * Copyright (c) 2009 by David Michael Betz.  All rights reserved.
 *
 */

#include "windows.h"

#include <stdio.h>
#include <stdarg.h>
#include "../common/db_packet.h"

static HANDLE hSerial;
static COMMTIMEOUTS timeouts;

int md_init(char *port)
{
    char fullPort[20];
    DCB state;
    
    sprintf(fullPort, "\\\\.\\%s", port);

    hSerial = CreateFile(
        fullPort,
        GENERIC_READ | GENERIC_WRITE,
        0,
        NULL,
        OPEN_EXISTING,
        0,
        NULL);

    if (hSerial == INVALID_HANDLE_VALUE)
        return VMFALSE;

    GetCommState(hSerial, &state);
    state.BaudRate = CBR_9600;
    state.fBinary = VMTRUE;
    state.fParity = VMFALSE;
    state.fOutxCtsFlow = VMFALSE;
    state.fOutxDsrFlow = VMFALSE;
    state.fDtrControl = DTR_CONTROL_DISABLE;
    state.fDsrSensitivity = VMFALSE;
    state.fTXContinueOnXoff = VMTRUE;
    state.fOutX = VMFALSE;
    state.fInX = VMFALSE;
    state.fErrorChar = VMFALSE;
    state.fNull = VMFALSE;
    state.fRtsControl = RTS_CONTROL_DISABLE;
    state.fAbortOnError = VMFALSE;
    state.ByteSize = 8;
    state.Parity = NOPARITY;
    state.StopBits = ONESTOPBIT;
    SetCommState(hSerial, &state);

    GetCommTimeouts(hSerial, &timeouts);
    timeouts.ReadIntervalTimeout = MAXDWORD;
    timeouts.ReadTotalTimeoutMultiplier = MAXDWORD;

    return VMTRUE;
}

void md_done(void)
{
    CloseHandle(hSerial);
}

int md_write(uint8_t *buf, int len)
{
    DWORD written;
    BOOL ok;
    ok = WriteFile(hSerial, (void *)buf, (DWORD)len, &written, NULL);
    return ok && (int)written == len;
}

int md_get(int timeout)
{
    unsigned char buf[1];
    DWORD read;
    BOOL ok;
    timeouts.ReadTotalTimeoutConstant = timeout * 1000;
    SetCommTimeouts(hSerial, &timeouts);
    ok = ReadFile(hSerial, (void *)buf, (DWORD)1, &read, NULL);
    return ok && read == 1 ? buf[0] : MD_TIMEOUT;
}
