// this version is for the PIC

option target = "tile"
option heap = "0"

// these must match the tile engine that has been built into the runtime
def screenw = 28
def screenh = 25
def tilew   = 6
def tileh   = 8

def worldw = screenw
def worldh = screenh

def generations = 40

def gp_right  = 0x01
def gp_left   = 0x02
def gp_down   = 0x04
def gp_up     = 0x08
def gp_start  = 0x10
def gp_select = 0x20
def gp_red    = 0x40
def gp_yellow = 0x80

def arrayw = 1 + worldw + 1 + worldw + 1
def arrayh = 1 + worldh + 1

def world1 = 1
def world2 = 1 + worldw + 1

dim world(arrayw, arrayh) as byte
dim thisx, nextx

dim tileMap(screenw, screenh) as byte
dim tiles(tilew, tileh*2) as byte
dim colorTable(256) as byte

def visit(x, y)
  for y1 = y-1 to y+1
    for x1 = x-1 to x+1
      if world(x1, y1) > 0 then
        visit = visit + 1
      end if
    next x1
  next y1
end def

sub clearall
  for y1 = 1 to arrayh
    for x1 = 1 to arrayw
      world(x1, y1) = 0
    next x1
  next y1
end sub

sub fill
  for y1 = 2 to worldh+1
    for x1 = 1 to worldw
      if rnd(4) = 3 then
        cell = 1
      else
        cell = 0
      end if
      world(thisx+x1, y1) = cell 
    next x1
  next y1
end sub

sub nextgen
  for y1 = 2 to worldh+1
    for x1 = 1 to worldw
      neighbors = visit(thisx+x1, y1)
      if world(thisx+x1, y1) = 0 then
        if neighbors = 3 then
          new = 1
        else
          new = 0
        end if
      else
        if neighbors = 3 or neighbors = 4 then
          new = 1
        else
          new = 0
        end if
      end if
      world(nextx+x1, y1) = new
    next x1
  next y1
end sub

sub showgen
  for y1 = 1 to worldh
    for x1 = 1 to worldw
      tileMap(x1, y1) = world(thisx+x1, 1+y1)
    next x1
  next y1
end sub

print "initializing tile map"
for y = 1 to screenh
  for x = 1 to screenw
    tileMap(x, y) = 0
  next x
next y

print "initializing tiles"
for y = 1 to tileh
  for x = 1 to tilew
    tiles(x, y) = 0x4f
    if x=1 or x=tilew or y=1 or y=tileh then
      tiles(x, tileh+y) = 0x4f
    else
      tiles(x, tileh+y) = 0x91
    end if
  next x
next y

print "initializing color table"
for x = 1 to 256
  colorTable(x) = x
next x

print "setting up graphics"
settilemap(tileMap)
settiles(tiles)
setcolortable(colorTable)

print "clearing worlds"
clearall

do

  rem initialize
  thisx = world1
  nextx = world2

  print "filling world"
  fill

  generation = 0

  do

    rem count the generation
    generation = generation + 1

    rem wait for vsync to display the current generation
    waitforvsync
    showgen
    
    rem compute the next generation
    nextgen
    
    rem swap worlds
    tmp = thisx
    thisx = nextx
    nextx = tmp

    rem determine if it's time to stop
    buttons = gamepad(0)
    done = not (buttons & gp_yellow) and (generation >= generations)

  loop until done

loop
