rem ------------------------------------------------------
rem A pong clone in XGS Basic
rem Created by Josh Hintze
rem Compatabile with XGS Basic v1.4
rem Copyright 2009 Nurve Networks LLC.
rem ------------------------------------------------------

rem Set the target as bitmap (not tile)
option target = "bitmap"

rem ------------------------------------------------------

dim ballx
dim bally
dim dirx
dim diry
dim player1_paddle
dim cpu_paddle
dim cpu_hesitation

dim sound_counter

dim game_paused
dim player_score
dim cpu_score
dim cpu_rand_hit_loc

def ball_width = 3
def ball_height = 4
def paddle_height = 15
def paddle_speed = 3

rem Gamepad direction defines
def gp_right  = 0x01
def gp_left   = 0x02
def gp_down   = 0x04
def gp_up     = 0x08
def gp_start  = 0x10
def gp_select = 0x20
def gp_red    = 0x40
def gp_yellow = 0x80

rem ------------------------------------------------------
rem Draws the playfield except for the middle line
rem ------------------------------------------------------

sub UpdateBall
    rem Erase the ball
    fillrect(ballx, bally, ball_width, ball_height, 0)
       
    rem Update the X and Y positions   
    ballx = ballx + dirx
    if ballx <= 11 then
        rem Check for the paddle collision
        if(bally < (player1_paddle - 5)) | (bally > (player1_paddle + paddle_height + 5)) then
            rem MISS
            ballx = 60
            bally = rnd(170) + 20
            game_paused = 1
            dirx = 2
            diry = -2 + rnd(4)            
            cpu_score = cpu_score + 1
        else
            rem HIT
            dirx = 2
            sound(500)
            sound_counter = 15
            cpu_hesitation = 30
            
            rem update y direction based on ball hit location
            if(bally < player1_paddle + paddle_height/2 - 3) then
                diry = -2
            else if(bally > player1_paddle + paddle_height/2 + 3) then    
                diry = 2
            else
                diry = 0
            end if
            
        end if
    else if ballx >= 148 then
        rem Check for the paddle collision with cpu
        if(bally < (cpu_paddle - 5)) | (bally > (cpu_paddle + paddle_height + 5)) then
            rem MISS
            ballx = 100
            bally = rnd(170) + 20
            game_paused = 1
            dirx = -2
            diry = -2 + rnd(4)       
            player_score = player_score + 1
        else
            rem HIT
            dirx = -2
            sound(700)
            sound_counter = 15
            cpu_hesitation = 20
            cpu_rand_hit_loc = rnd(15)
            
            rem update y direction based on ball hit location
            if(bally < cpu_paddle + paddle_height/2 - 3) then
                diry = -2
            else if(bally > cpu_paddle + paddle_height/2 + 3) then    
                diry = 2
            else
                diry = 0
            end if

        end if
    end if
    
    bally = bally + diry
    if bally > 191 then
        bally = 191
        diry = -2
    else if bally < 0 then
        bally = 0
        diry = 2
    end if
    
    
    rem Draw the ball
    fillrect(ballx, bally, ball_width, ball_height, 1)
    
end sub

rem ------------------------------------------------------


rem ------------------------------------------------------
rem Checks gamepad for input and moves the paddles
rem ------------------------------------------------------

sub UpdatePaddles

    rem Erase the previous paddles
    vline(10, player1_paddle, player1_paddle + paddle_height, 0)
    vline(150, cpu_paddle, cpu_paddle + paddle_height, 0)

    rem Check for player input
    buttons = gamepad(0)
    if(buttons & gp_up) then
        player1_paddle = player1_paddle - paddle_speed
    else if(buttons & gp_down) then
        player1_paddle = player1_paddle + paddle_speed
    end if
    
    rem Check paddle limits paddle 1
    if(player1_paddle < 0) then
        player1_paddle = 0
    end if
    
    if(player1_paddle > 191 - paddle_height) then
        player1_paddle = 191 - paddle_height
    end if
    
    rem Computer AI 
    cpu_dy = 0
    
    if(cpu_hesitation > 0) then
        cpu_hesitation = cpu_hesitation - 1
    else     
        rem Check ball direction    
        if(dirx < 0) then
            rem Ball is traveling away so mirror the players movements
            if(player1_paddle < cpu_paddle) then
                cpu_dy = -1
            else if(player1_paddle > cpu_paddle) then
                cpu_dy = 1
            end if
        else
            rem Ball is traveling towards our paddle
            if(bally < cpu_paddle + cpu_rand_hit_loc) then
                cpu_dy = -2
            else if(bally > cpu_paddle + cpu_rand_hit_loc) then
                cpu_dy = 2
            end if
        end if
    end if    
    
    
    cpu_paddle = cpu_paddle + cpu_dy
    
    rem Check paddle limits cpu paddle
    if(cpu_paddle < 0) then
        cpu_paddle = 0
    end if

    if(cpu_paddle > 191 - paddle_height) then
        cpu_paddle = 191 - paddle_height
    end if
        

    rem Now draw the paddles
    vline(10, player1_paddle, player1_paddle + paddle_height, 3)
    vline(150, cpu_paddle, cpu_paddle + paddle_height, 3)


end sub

rem ------------------------------------------------------
rem Displays the score up to 5, then winner at 5
rem ------------------------------------------------------

sub DisplayScore
         
    rem Draw the player_score
    position(65, 5)
    display player_score

    rem Draw the cpu_score
    position(85, 5)
    display cpu_score

    rem Display winner loser
    if player_score = 5 then
        position(20, 20)
        display "winner"
        position(120, 20)
        display "loser"
    else if cpu_score = 5 then
        position(20, 20)
        display "loser"
        position(120, 20)
        display "winner"
    end if
        
end sub



rem ------------------------------------------------------
rem Main Code
rem ------------------------------------------------------

do
    rem ------Initialize------
    ballx = 70
    bally = 96
    dirx = 2
    diry = 2
    player1_paddle = 76
    cpu_paddle = 76
    game_paused = 1
    cpu_hesitation = 0
    player_score = 0
    cpu_score = 0
    cpu_rand_hit_loc = rnd(15)
    
    rem ------Title Screen------
    clear(0)
    vline(0, 0, 191, 3)
    vline(159, 0, 191, 3)
    hline(0, 159, 0, 3)
    hline(0, 159, 191, 3)
    text(55,20, "XGS PONG")
    text(20,40, "Powered by XGS BASIC")
    text(20,160, "Press Start to Play")
    
    do while not (gamepad(0) & gp_start)

    loop
    
    clear(0)

    rem ------Game Loop------
    do
    
        rem Wait for vsync to update display
        waitforvsync
     
        rem Update the ball position
        UpdateBall

        rem Redraw the vertical divider line
        vline(80, 0, 191, 0)
        vline_starty = 0
        for i = 0 to 32
            vline(80, vline_starty, vline_starty+3, 2)
            vline_starty = vline_starty + 6
        next i

        rem Update the paddle positions based on gamepads
        UpdatePaddles

        rem Show scores
        DisplayScore

        rem Check for sound playing
        if (sound_counter <> 0) then
            sound_counter = sound_counter - 1
            if(sound_counter = 0) then
                sound(0)
            end if
        end if
        
        rem See if game is paused
        do while (game_paused)
            if(gamepad(0) & gp_yellow) then
                game_paused = 0
                if (player_score = 5) | (cpu_score = 5) then
                    player_score = 0
                    cpu_score = 0
                    player1_paddle = 76
                    cpu_paddle = 76
                    clear(0)
                end if

            end if
        loop
        
    
    loop

loop

rem ------------------------------------------------------
