/* db_vmpic.c - a simple virtual machine for the XGS PIC system
 *
 * Copyright (c) 2009 by David Michael Betz.  All rights reserved.
 *
 */

/* needs to be built with:

db_vmpic.c
db_vmfcnpic.c
db_picuart.c
db_text.c
../common/db_crctab.c
../common/db_packet.c
../runtime/db_vmfcn.c
../runtime/db_vmflash.c
../runtime/db_vmheap.c
../runtime/db_vmimage.c
../runtime/db_vmint.c
../runtime/db_vmmain.c
FSIO.c
SD-SPI.c
XGS_PIC_GAMEPAD_DRV_V010.c
XGS_PIC_GFX_DRV_V010.c
XGS_PIC_SOUND_DRV_V010.c
XGS_PIC_SYSTEM_V010.c
XGS_PIC_UART_DRV_V010.c
XGS_PIC_NTSC_160_192_2_V010.s
XGS_PIC_NTSC_TILE2_V010.s

In order to make serial downloads reliable, it is necessary to change the following definition in XGS_PIC_UART_DRV_V010.h:

  #define UART1_RX_BUFFER_SIZE	135

The receive buffer needs to be large enough to hold a 5+128+2=135 byte packet.

In MPLAB under Project -> Build Options -> Project
and then under the "MPLAB LINK30" tab, set the heap size to 512. 

*/

#include "XGS_PIC_SYSTEM_V010.h"			
#include "XGS_PIC_GFX_DRV_V010.h"
#include "XGS_PIC_UART_DRV_V010.h"
#include "XGS_PIC_GAMEPAD_DRV_V010.h"
#include "XGS_PIC_SOUND_DRV_V010.h"

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h>
#include "../runtime/db_vm.h"
#include "../common/db_packet.h"
#include "db_text.h"

void VM_sysinit(void)
{
    SYS_ConfigureClock(FCY_NTSC);
    Gamepad_Init();
    SND_Init();
    FSInit();
}

VMFILE *VM_fopen(const char *name, const char *mode)
{
    return (VMFILE *)FSfopen(name, mode);
}

int VM_fclose(VMFILE *fp)
{
    return FSfclose((FSFILE *)fp);
}

size_t VM_fread(void *buf, size_t count, VMFILE *fp)
{
    return FSfread(buf, 1, count, (FSFILE *)fp);
}

size_t VM_fwrite(const void *buf, size_t count, VMFILE *fp)
{
    return FSfwrite(buf, 1, count, (FSFILE *)fp);
}

int VM_fseek(VMFILE *fp, long offset, int whence)
{
    return FSfseek((FSFILE *)fp, offset, whence);
}

long VM_ftell(VMFILE *fp)
{
    return FSftell((FSFILE *)fp);
}

